# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .usage import (
    UsageResource,
    AsyncUsageResource,
    UsageResourceWithRawResponse,
    AsyncUsageResourceWithRawResponse,
    UsageResourceWithStreamingResponse,
    AsyncUsageResourceWithStreamingResponse,
)
from .invoice import (
    InvoiceResource,
    AsyncInvoiceResource,
    InvoiceResourceWithRawResponse,
    AsyncInvoiceResourceWithRawResponse,
    InvoiceResourceWithStreamingResponse,
    AsyncInvoiceResourceWithStreamingResponse,
)
from .future_update import (
    FutureUpdateResource,
    AsyncFutureUpdateResource,
    FutureUpdateResourceWithRawResponse,
    AsyncFutureUpdateResourceWithRawResponse,
    FutureUpdateResourceWithStreamingResponse,
    AsyncFutureUpdateResourceWithStreamingResponse,
)
from .subscriptions import (
    SubscriptionsResource,
    AsyncSubscriptionsResource,
    SubscriptionsResourceWithRawResponse,
    AsyncSubscriptionsResourceWithRawResponse,
    SubscriptionsResourceWithStreamingResponse,
    AsyncSubscriptionsResourceWithStreamingResponse,
)

__all__ = [
    "FutureUpdateResource",
    "AsyncFutureUpdateResource",
    "FutureUpdateResourceWithRawResponse",
    "AsyncFutureUpdateResourceWithRawResponse",
    "FutureUpdateResourceWithStreamingResponse",
    "AsyncFutureUpdateResourceWithStreamingResponse",
    "UsageResource",
    "AsyncUsageResource",
    "UsageResourceWithRawResponse",
    "AsyncUsageResourceWithRawResponse",
    "UsageResourceWithStreamingResponse",
    "AsyncUsageResourceWithStreamingResponse",
    "InvoiceResource",
    "AsyncInvoiceResource",
    "InvoiceResourceWithRawResponse",
    "AsyncInvoiceResourceWithRawResponse",
    "InvoiceResourceWithStreamingResponse",
    "AsyncInvoiceResourceWithStreamingResponse",
    "SubscriptionsResource",
    "AsyncSubscriptionsResource",
    "SubscriptionsResourceWithRawResponse",
    "AsyncSubscriptionsResourceWithRawResponse",
    "SubscriptionsResourceWithStreamingResponse",
    "AsyncSubscriptionsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["UsageChargeUsageResponse", "Data", "DataUsageCharged"]


class DataUsageCharged(BaseModel):
    """A single usage item that was charged."""

    feature_id: Optional[str] = FieldInfo(alias="featureId", default=None)
    """The feature ID for which usage was charged"""

    usage_amount: float = FieldInfo(alias="usageAmount")
    """The number of units charged"""


class Data(BaseModel):
    """
    Result of charging subscription usage including the billing period and charged items.
    """

    invoice_billing_id: Optional[str] = FieldInfo(alias="invoiceBillingId", default=None)
    """The invoice ID in the billing integration"""

    period_end: datetime = FieldInfo(alias="periodEnd")
    """End of the usage billing period"""

    period_start: datetime = FieldInfo(alias="periodStart")
    """Start of the usage billing period"""

    subscription_id: str = FieldInfo(alias="subscriptionId")
    """The subscription ID for which usage was charged"""

    usage_charged: List[DataUsageCharged] = FieldInfo(alias="usageCharged")
    """Usage items that were charged"""


class UsageChargeUsageResponse(BaseModel):
    """Response object"""

    data: Data
    """
    Result of charging subscription usage including the billing period and charged
    items.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["SubscriptionImportParams", "Subscription"]


class SubscriptionImportParams(TypedDict, total=False):
    subscriptions: Required[Iterable[Subscription]]
    """List of subscription objects to import"""

    integration_id: Annotated[Optional[str], PropertyInfo(alias="integrationId")]
    """Integration ID to use for importing subscriptions"""


class Subscription(TypedDict, total=False):
    id: Required[str]
    """Subscription ID"""

    customer_id: Required[Annotated[str, PropertyInfo(alias="customerId")]]
    """Customer ID"""

    plan_id: Required[Annotated[str, PropertyInfo(alias="planId")]]
    """Plan ID"""

    billing_id: Annotated[Optional[str], PropertyInfo(alias="billingId")]
    """Billing ID"""

    end_date: Annotated[Union[str, datetime, None], PropertyInfo(alias="endDate", format="iso8601")]
    """Subscription end date"""

    metadata: Dict[str, str]
    """Additional metadata for the subscription"""

    resource_id: Annotated[Optional[str], PropertyInfo(alias="resourceId")]
    """Resource ID"""

    start_date: Annotated[Union[str, datetime], PropertyInfo(alias="startDate", format="iso8601")]
    """Subscription start date"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from stigg import Stigg, AsyncStigg
from tests.utils import assert_matches_type
from stigg._utils import parse_datetime
from stigg.types.v1 import (
    Coupon,
    CouponListResponse,
)
from stigg.pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCoupons:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Stigg) -> None:
        coupon = client.v1.coupons.create(
            id="id",
            amounts_off=[
                {
                    "amount": 0,
                    "currency": "usd",
                }
            ],
            description="description",
            duration_in_months=1,
            metadata={"foo": "string"},
            name="name",
            percent_off=1,
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Stigg) -> None:
        response = client.v1.coupons.with_raw_response.create(
            id="id",
            amounts_off=[
                {
                    "amount": 0,
                    "currency": "usd",
                }
            ],
            description="description",
            duration_in_months=1,
            metadata={"foo": "string"},
            name="name",
            percent_off=1,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Stigg) -> None:
        with client.v1.coupons.with_streaming_response.create(
            id="id",
            amounts_off=[
                {
                    "amount": 0,
                    "currency": "usd",
                }
            ],
            description="description",
            duration_in_months=1,
            metadata={"foo": "string"},
            name="name",
            percent_off=1,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Stigg) -> None:
        coupon = client.v1.coupons.retrieve(
            "x",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Stigg) -> None:
        response = client.v1.coupons.with_raw_response.retrieve(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Stigg) -> None:
        with client.v1.coupons.with_streaming_response.retrieve(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.coupons.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Stigg) -> None:
        coupon = client.v1.coupons.list()
        assert_matches_type(SyncMyCursorIDPage[CouponListResponse], coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Stigg) -> None:
        coupon = client.v1.coupons.list(
            id="id",
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            created_at={
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            limit=1,
            status="status",
            type="FIXED",
        )
        assert_matches_type(SyncMyCursorIDPage[CouponListResponse], coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Stigg) -> None:
        response = client.v1.coupons.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = response.parse()
        assert_matches_type(SyncMyCursorIDPage[CouponListResponse], coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Stigg) -> None:
        with client.v1.coupons.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = response.parse()
            assert_matches_type(SyncMyCursorIDPage[CouponListResponse], coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_archive_coupon(self, client: Stigg) -> None:
        coupon = client.v1.coupons.archive_coupon(
            "x",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_archive_coupon(self, client: Stigg) -> None:
        response = client.v1.coupons.with_raw_response.archive_coupon(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_archive_coupon(self, client: Stigg) -> None:
        with client.v1.coupons.with_streaming_response.archive_coupon(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_archive_coupon(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.coupons.with_raw_response.archive_coupon(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_coupon(self, client: Stigg) -> None:
        coupon = client.v1.coupons.update_coupon(
            id="x",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_coupon_with_all_params(self, client: Stigg) -> None:
        coupon = client.v1.coupons.update_coupon(
            id="x",
            description="description",
            metadata={"foo": "string"},
            name="name",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update_coupon(self, client: Stigg) -> None:
        response = client.v1.coupons.with_raw_response.update_coupon(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update_coupon(self, client: Stigg) -> None:
        with client.v1.coupons.with_streaming_response.update_coupon(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update_coupon(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.coupons.with_raw_response.update_coupon(
                id="",
            )


class TestAsyncCoupons:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncStigg) -> None:
        coupon = await async_client.v1.coupons.create(
            id="id",
            amounts_off=[
                {
                    "amount": 0,
                    "currency": "usd",
                }
            ],
            description="description",
            duration_in_months=1,
            metadata={"foo": "string"},
            name="name",
            percent_off=1,
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.coupons.with_raw_response.create(
            id="id",
            amounts_off=[
                {
                    "amount": 0,
                    "currency": "usd",
                }
            ],
            description="description",
            duration_in_months=1,
            metadata={"foo": "string"},
            name="name",
            percent_off=1,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = await response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.coupons.with_streaming_response.create(
            id="id",
            amounts_off=[
                {
                    "amount": 0,
                    "currency": "usd",
                }
            ],
            description="description",
            duration_in_months=1,
            metadata={"foo": "string"},
            name="name",
            percent_off=1,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = await response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncStigg) -> None:
        coupon = await async_client.v1.coupons.retrieve(
            "x",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.coupons.with_raw_response.retrieve(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = await response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.coupons.with_streaming_response.retrieve(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = await response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.coupons.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncStigg) -> None:
        coupon = await async_client.v1.coupons.list()
        assert_matches_type(AsyncMyCursorIDPage[CouponListResponse], coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncStigg) -> None:
        coupon = await async_client.v1.coupons.list(
            id="id",
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            created_at={
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            limit=1,
            status="status",
            type="FIXED",
        )
        assert_matches_type(AsyncMyCursorIDPage[CouponListResponse], coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.coupons.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = await response.parse()
        assert_matches_type(AsyncMyCursorIDPage[CouponListResponse], coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.coupons.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = await response.parse()
            assert_matches_type(AsyncMyCursorIDPage[CouponListResponse], coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_archive_coupon(self, async_client: AsyncStigg) -> None:
        coupon = await async_client.v1.coupons.archive_coupon(
            "x",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_archive_coupon(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.coupons.with_raw_response.archive_coupon(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = await response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_archive_coupon(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.coupons.with_streaming_response.archive_coupon(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = await response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_archive_coupon(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.coupons.with_raw_response.archive_coupon(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_coupon(self, async_client: AsyncStigg) -> None:
        coupon = await async_client.v1.coupons.update_coupon(
            id="x",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_coupon_with_all_params(self, async_client: AsyncStigg) -> None:
        coupon = await async_client.v1.coupons.update_coupon(
            id="x",
            description="description",
            metadata={"foo": "string"},
            name="name",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update_coupon(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.coupons.with_raw_response.update_coupon(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = await response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update_coupon(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.coupons.with_streaming_response.update_coupon(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = await response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update_coupon(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.coupons.with_raw_response.update_coupon(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["CouponCreateParams", "AmountsOff"]


class CouponCreateParams(TypedDict, total=False):
    id: Required[str]
    """The unique identifier for the entity"""

    amounts_off: Required[Annotated[Optional[Iterable[AmountsOff]], PropertyInfo(alias="amountsOff")]]
    """Fixed amount discounts in different currencies"""

    description: Required[Optional[str]]
    """Description of the coupon"""

    duration_in_months: Required[Annotated[Optional[int], PropertyInfo(alias="durationInMonths")]]
    """Duration of the coupon validity in months"""

    metadata: Required[Optional[Dict[str, str]]]
    """Metadata associated with the entity"""

    name: Required[str]
    """Name of the coupon"""

    percent_off: Required[Annotated[Optional[float], PropertyInfo(alias="percentOff")]]
    """Percentage discount off the original price"""


class AmountsOff(TypedDict, total=False):
    """Monetary amount with currency"""

    amount: Required[float]
    """The price amount"""

    currency: Required[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ]
    """The price currency"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "SubscriptionProvisionResponse",
    "Data",
    "DataEntitlement",
    "DataEntitlementUnionObjectVariant0",
    "DataEntitlementUnionObjectVariant0Feature",
    "DataEntitlementUnionObjectVariant1",
    "DataEntitlementUnionObjectVariant1Currency",
    "DataSubscription",
    "DataSubscriptionPrice",
    "DataSubscriptionPricePrice",
    "DataSubscriptionPriceTier",
    "DataSubscriptionPriceTierFlatPrice",
    "DataSubscriptionPriceTierUnitPrice",
]


class DataEntitlementUnionObjectVariant0Feature(BaseModel):
    display_name: str = FieldInfo(alias="displayName")
    """The human-readable name of the entitlement, shown in UI elements."""

    feature_status: Literal["NEW", "SUSPENDED", "ACTIVE"] = FieldInfo(alias="featureStatus")
    """The current status of the feature."""

    feature_type: Literal["BOOLEAN", "NUMBER", "ENUM"] = FieldInfo(alias="featureType")
    """The type of feature associated with the entitlement."""

    ref_id: str = FieldInfo(alias="refId")
    """The unique reference ID of the entitlement."""


class DataEntitlementUnionObjectVariant0(BaseModel):
    access_denied_reason: Optional[
        Literal[
            "FeatureNotFound",
            "CustomerNotFound",
            "CustomerIsArchived",
            "CustomerResourceNotFound",
            "NoActiveSubscription",
            "NoFeatureEntitlementInSubscription",
            "RequestedUsageExceedingLimit",
            "RequestedValuesMismatch",
            "BudgetExceeded",
            "Unknown",
            "FeatureTypeMismatch",
            "Revoked",
            "InsufficientCredits",
            "EntitlementNotFound",
        ]
    ] = FieldInfo(alias="accessDeniedReason", default=None)

    is_granted: bool = FieldInfo(alias="isGranted")

    type: Literal["FEATURE"]

    current_usage: Optional[float] = FieldInfo(alias="currentUsage", default=None)

    entitlement_updated_at: Optional[datetime] = FieldInfo(alias="entitlementUpdatedAt", default=None)
    """Timestamp of the last update to the entitlement grant or configuration."""

    feature: Optional[DataEntitlementUnionObjectVariant0Feature] = None

    has_unlimited_usage: Optional[bool] = FieldInfo(alias="hasUnlimitedUsage", default=None)

    reset_period: Optional[Literal["YEAR", "MONTH", "WEEK", "DAY", "HOUR"]] = FieldInfo(
        alias="resetPeriod", default=None
    )

    usage_limit: Optional[float] = FieldInfo(alias="usageLimit", default=None)

    usage_period_anchor: Optional[datetime] = FieldInfo(alias="usagePeriodAnchor", default=None)
    """
    The anchor for calculating the usage period for metered entitlements with a
    reset period configured
    """

    usage_period_end: Optional[datetime] = FieldInfo(alias="usagePeriodEnd", default=None)
    """
    The end date of the usage period for metered entitlements with a reset period
    configured
    """

    usage_period_start: Optional[datetime] = FieldInfo(alias="usagePeriodStart", default=None)
    """
    The start date of the usage period for metered entitlements with a reset period
    configured
    """

    valid_until: Optional[datetime] = FieldInfo(alias="validUntil", default=None)
    """The next time the entitlement should be recalculated"""


class DataEntitlementUnionObjectVariant1Currency(BaseModel):
    """The currency associated with a credit entitlement."""

    currency_id: str = FieldInfo(alias="currencyId")
    """The unique identifier of the custom currency."""

    display_name: str = FieldInfo(alias="displayName")
    """The display name of the currency."""

    additional_meta_data: Optional[object] = FieldInfo(alias="additionalMetaData", default=None)
    """Additional metadata associated with the currency."""

    description: Optional[str] = None
    """A description of the currency."""

    unit_plural: Optional[str] = FieldInfo(alias="unitPlural", default=None)
    """The plural form of the currency unit."""

    unit_singular: Optional[str] = FieldInfo(alias="unitSingular", default=None)
    """The singular form of the currency unit."""


class DataEntitlementUnionObjectVariant1(BaseModel):
    access_denied_reason: Optional[
        Literal[
            "FeatureNotFound",
            "CustomerNotFound",
            "CustomerIsArchived",
            "CustomerResourceNotFound",
            "NoActiveSubscription",
            "NoFeatureEntitlementInSubscription",
            "RequestedUsageExceedingLimit",
            "RequestedValuesMismatch",
            "BudgetExceeded",
            "Unknown",
            "FeatureTypeMismatch",
            "Revoked",
            "InsufficientCredits",
            "EntitlementNotFound",
        ]
    ] = FieldInfo(alias="accessDeniedReason", default=None)

    currency: DataEntitlementUnionObjectVariant1Currency
    """The currency associated with a credit entitlement."""

    current_usage: float = FieldInfo(alias="currentUsage")

    is_granted: bool = FieldInfo(alias="isGranted")

    type: Literal["CREDIT"]

    usage_limit: float = FieldInfo(alias="usageLimit")

    usage_updated_at: datetime = FieldInfo(alias="usageUpdatedAt")
    """Timestamp of the last update to the credit usage."""

    entitlement_updated_at: Optional[datetime] = FieldInfo(alias="entitlementUpdatedAt", default=None)
    """Timestamp of the last update to the entitlement grant or configuration."""

    usage_period_end: Optional[datetime] = FieldInfo(alias="usagePeriodEnd", default=None)
    """The end date of the current billing period for recurring credit grants."""

    valid_until: Optional[datetime] = FieldInfo(alias="validUntil", default=None)
    """The next time the entitlement should be recalculated"""


DataEntitlement: TypeAlias = Union[DataEntitlementUnionObjectVariant0, DataEntitlementUnionObjectVariant1]


class DataSubscriptionPricePrice(BaseModel):
    """Override price amount"""

    amount: Optional[float] = None
    """The price amount"""

    billing_country_code: Optional[str] = FieldInfo(alias="billingCountryCode", default=None)
    """The billing country code of the price"""

    currency: Optional[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ] = None
    """The price currency"""


class DataSubscriptionPriceTierFlatPrice(BaseModel):
    """The flat fee price of the price tier"""

    amount: Optional[float] = None
    """The price amount"""

    billing_country_code: Optional[str] = FieldInfo(alias="billingCountryCode", default=None)
    """The billing country code of the price"""

    currency: Optional[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ] = None
    """The price currency"""


class DataSubscriptionPriceTierUnitPrice(BaseModel):
    """The unit price of the price tier"""

    amount: Optional[float] = None
    """The price amount"""

    billing_country_code: Optional[str] = FieldInfo(alias="billingCountryCode", default=None)
    """The billing country code of the price"""

    currency: Optional[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ] = None
    """The price currency"""


class DataSubscriptionPriceTier(BaseModel):
    flat_price: Optional[DataSubscriptionPriceTierFlatPrice] = FieldInfo(alias="flatPrice", default=None)
    """The flat fee price of the price tier"""

    unit_price: Optional[DataSubscriptionPriceTierUnitPrice] = FieldInfo(alias="unitPrice", default=None)
    """The unit price of the price tier"""

    up_to: Optional[float] = FieldInfo(alias="upTo", default=None)
    """The up to quantity of the price tier"""


class DataSubscriptionPrice(BaseModel):
    addon_id: Optional[str] = FieldInfo(alias="addonId", default=None)
    """Addon identifier for the price override"""

    base_charge: Optional[bool] = FieldInfo(alias="baseCharge", default=None)
    """Whether this is a base charge override"""

    block_size: Optional[float] = FieldInfo(alias="blockSize", default=None)
    """Block size for pricing"""

    feature_id: Optional[str] = FieldInfo(alias="featureId", default=None)
    """Feature identifier for the price override"""

    price: Optional[DataSubscriptionPricePrice] = None
    """Override price amount"""

    tiers: Optional[List[DataSubscriptionPriceTier]] = None
    """Pricing tiers configuration"""


class DataSubscription(BaseModel):
    """Created subscription (when status is SUCCESS)"""

    id: str
    """Subscription ID"""

    billing_id: Optional[str] = FieldInfo(alias="billingId", default=None)
    """Billing ID"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Created at"""

    customer_id: str = FieldInfo(alias="customerId")
    """Customer ID"""

    payment_collection: Literal["NOT_REQUIRED", "PROCESSING", "FAILED", "ACTION_REQUIRED"] = FieldInfo(
        alias="paymentCollection"
    )
    """Payment collection"""

    plan_id: str = FieldInfo(alias="planId")
    """Plan ID"""

    pricing_type: Literal["FREE", "PAID", "CUSTOM"] = FieldInfo(alias="pricingType")
    """Pricing type"""

    start_date: datetime = FieldInfo(alias="startDate")
    """Subscription start date"""

    status: Literal["PAYMENT_PENDING", "ACTIVE", "EXPIRED", "IN_TRIAL", "CANCELED", "NOT_STARTED"]
    """Subscription status"""

    cancellation_date: Optional[datetime] = FieldInfo(alias="cancellationDate", default=None)
    """Subscription cancellation date"""

    cancel_reason: Optional[
        Literal[
            "UPGRADE_OR_DOWNGRADE",
            "CANCELLED_BY_BILLING",
            "EXPIRED",
            "DETACH_BILLING",
            "TRIAL_ENDED",
            "Immediate",
            "TRIAL_CONVERTED",
            "PENDING_PAYMENT_EXPIRED",
            "ScheduledCancellation",
            "CustomerArchived",
            "AutoCancellationRule",
        ]
    ] = FieldInfo(alias="cancelReason", default=None)
    """Subscription cancel reason"""

    current_billing_period_end: Optional[datetime] = FieldInfo(alias="currentBillingPeriodEnd", default=None)
    """End of the current billing period"""

    current_billing_period_start: Optional[datetime] = FieldInfo(alias="currentBillingPeriodStart", default=None)
    """Start of the current billing period"""

    effective_end_date: Optional[datetime] = FieldInfo(alias="effectiveEndDate", default=None)
    """Subscription effective end date"""

    end_date: Optional[datetime] = FieldInfo(alias="endDate", default=None)
    """Subscription end date"""

    metadata: Optional[Dict[str, str]] = None
    """Additional metadata for the subscription"""

    paying_customer_id: Optional[str] = FieldInfo(alias="payingCustomerId", default=None)
    """Paying customer ID for delegated billing"""

    payment_collection_method: Optional[Literal["CHARGE", "INVOICE", "NONE"]] = FieldInfo(
        alias="paymentCollectionMethod", default=None
    )
    """The method used to collect payments for a subscription"""

    prices: Optional[List[DataSubscriptionPrice]] = None

    resource_id: Optional[str] = FieldInfo(alias="resourceId", default=None)
    """Resource ID"""

    trial_end_date: Optional[datetime] = FieldInfo(alias="trialEndDate", default=None)
    """Subscription trial end date"""


class Data(BaseModel):
    """Provisioning result with status and subscription or checkout URL."""

    id: str
    """Unique identifier for the provisioned subscription"""

    entitlements: Optional[List[DataEntitlement]] = None

    status: Literal["SUCCESS", "PAYMENT_REQUIRED"]
    """Provision status: SUCCESS or PAYMENT_REQUIRED"""

    subscription: Optional[DataSubscription] = None
    """Created subscription (when status is SUCCESS)"""

    checkout_billing_id: Optional[str] = FieldInfo(alias="checkoutBillingId", default=None)
    """Checkout billing ID when payment is required"""

    checkout_url: Optional[str] = FieldInfo(alias="checkoutUrl", default=None)
    """URL to complete payment when PAYMENT_REQUIRED"""

    is_scheduled: Optional[bool] = FieldInfo(alias="isScheduled", default=None)
    """Whether the subscription is scheduled for future activation"""


class SubscriptionProvisionResponse(BaseModel):
    """Response object"""

    data: Data
    """Provisioning result with status and subscription or checkout URL."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["SubscriptionDelegateParams"]


class SubscriptionDelegateParams(TypedDict, total=False):
    target_customer_id: Required[Annotated[str, PropertyInfo(alias="targetCustomerId")]]
    """
    The unique identifier of the customer who will assume payment responsibility for
    this subscription. This customer must already exist in your Stigg account and
    have a valid payment method if the subscription requires payment.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import feature_list_features_params, feature_create_feature_params, feature_update_feature_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.v1.feature import Feature
from ...types.v1.feature_list_features_response import FeatureListFeaturesResponse

__all__ = ["FeaturesResource", "AsyncFeaturesResource"]


class FeaturesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FeaturesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return FeaturesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FeaturesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return FeaturesResourceWithStreamingResponse(self)

    def archive_feature(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feature:
        """
        Archives a feature, preventing it from being used in new entitlements.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/features/{id}/archive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feature,
        )

    def create_feature(
        self,
        *,
        id: str,
        display_name: str,
        feature_type: Literal["BOOLEAN", "NUMBER", "ENUM"],
        description: str | Omit = omit,
        enum_configuration: Iterable[feature_create_feature_params.EnumConfiguration] | Omit = omit,
        feature_status: Literal["NEW", "SUSPENDED", "ACTIVE"] | Omit = omit,
        feature_units: str | Omit = omit,
        feature_units_plural: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        meter_type: Literal["None", "FLUCTUATING", "INCREMENTAL"] | Omit = omit,
        unit_transformation: Optional[feature_create_feature_params.UnitTransformation] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feature:
        """
        Creates a new feature with the specified type, metering, and configuration.

        Args:
          id: The unique identifier for the feature

          display_name: The display name for the feature

          feature_type: The type of the feature

          description: The description for the feature

          enum_configuration: The configuration data for the feature

          feature_status: The status of the feature

          feature_units: The units for the feature

          feature_units_plural: The plural units for the feature

          metadata: The additional metadata for the feature

          meter_type: The meter type for the feature

          unit_transformation: Unit transformation to be applied to the reported usage

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/features",
            body=maybe_transform(
                {
                    "id": id,
                    "display_name": display_name,
                    "feature_type": feature_type,
                    "description": description,
                    "enum_configuration": enum_configuration,
                    "feature_status": feature_status,
                    "feature_units": feature_units,
                    "feature_units_plural": feature_units_plural,
                    "metadata": metadata,
                    "meter_type": meter_type,
                    "unit_transformation": unit_transformation,
                },
                feature_create_feature_params.FeatureCreateFeatureParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feature,
        )

    def list_features(
        self,
        *,
        id: str | Omit = omit,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: feature_list_features_params.CreatedAt | Omit = omit,
        feature_type: str | Omit = omit,
        limit: int | Omit = omit,
        meter_type: str | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncMyCursorIDPage[FeatureListFeaturesResponse]:
        """
        Retrieves a paginated list of features in the environment.

        Args:
          id: Filter by entity ID

          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          feature_type: Filter by feature type. Supports comma-separated values for multiple types

          limit: Maximum number of items to return

          meter_type: Filter by meter type. Supports comma-separated values for multiple types

          status: Filter by feature status. Supports comma-separated values for multiple statuses

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/features",
            page=SyncMyCursorIDPage[FeatureListFeaturesResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "feature_type": feature_type,
                        "limit": limit,
                        "meter_type": meter_type,
                        "status": status,
                    },
                    feature_list_features_params.FeatureListFeaturesParams,
                ),
            ),
            model=FeatureListFeaturesResponse,
        )

    def retrieve_feature(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feature:
        """
        Retrieves a feature by its unique identifier.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/v1/features/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feature,
        )

    def unarchive_feature(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feature:
        """
        Restores an archived feature, allowing it to be used in entitlements again.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/features/{id}/unarchive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feature,
        )

    def update_feature(
        self,
        id: str,
        *,
        description: str | Omit = omit,
        display_name: str | Omit = omit,
        enum_configuration: Iterable[feature_update_feature_params.EnumConfiguration] | Omit = omit,
        feature_units: str | Omit = omit,
        feature_units_plural: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        meter: feature_update_feature_params.Meter | Omit = omit,
        unit_transformation: Optional[feature_update_feature_params.UnitTransformation] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feature:
        """
        Updates an existing feature's properties such as display name, description, and
        configuration.

        Args:
          description: The description for the feature

          display_name: The display name for the feature

          enum_configuration: The configuration data for the feature

          feature_units: The units for the feature

          feature_units_plural: The plural units for the feature

          metadata: The additional metadata for the feature

          unit_transformation: Unit transformation to be applied to the reported usage

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/api/v1/features/{id}",
            body=maybe_transform(
                {
                    "description": description,
                    "display_name": display_name,
                    "enum_configuration": enum_configuration,
                    "feature_units": feature_units,
                    "feature_units_plural": feature_units_plural,
                    "metadata": metadata,
                    "meter": meter,
                    "unit_transformation": unit_transformation,
                },
                feature_update_feature_params.FeatureUpdateFeatureParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feature,
        )


class AsyncFeaturesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFeaturesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFeaturesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFeaturesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncFeaturesResourceWithStreamingResponse(self)

    async def archive_feature(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feature:
        """
        Archives a feature, preventing it from being used in new entitlements.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/features/{id}/archive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feature,
        )

    async def create_feature(
        self,
        *,
        id: str,
        display_name: str,
        feature_type: Literal["BOOLEAN", "NUMBER", "ENUM"],
        description: str | Omit = omit,
        enum_configuration: Iterable[feature_create_feature_params.EnumConfiguration] | Omit = omit,
        feature_status: Literal["NEW", "SUSPENDED", "ACTIVE"] | Omit = omit,
        feature_units: str | Omit = omit,
        feature_units_plural: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        meter_type: Literal["None", "FLUCTUATING", "INCREMENTAL"] | Omit = omit,
        unit_transformation: Optional[feature_create_feature_params.UnitTransformation] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feature:
        """
        Creates a new feature with the specified type, metering, and configuration.

        Args:
          id: The unique identifier for the feature

          display_name: The display name for the feature

          feature_type: The type of the feature

          description: The description for the feature

          enum_configuration: The configuration data for the feature

          feature_status: The status of the feature

          feature_units: The units for the feature

          feature_units_plural: The plural units for the feature

          metadata: The additional metadata for the feature

          meter_type: The meter type for the feature

          unit_transformation: Unit transformation to be applied to the reported usage

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/features",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "display_name": display_name,
                    "feature_type": feature_type,
                    "description": description,
                    "enum_configuration": enum_configuration,
                    "feature_status": feature_status,
                    "feature_units": feature_units,
                    "feature_units_plural": feature_units_plural,
                    "metadata": metadata,
                    "meter_type": meter_type,
                    "unit_transformation": unit_transformation,
                },
                feature_create_feature_params.FeatureCreateFeatureParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feature,
        )

    def list_features(
        self,
        *,
        id: str | Omit = omit,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: feature_list_features_params.CreatedAt | Omit = omit,
        feature_type: str | Omit = omit,
        limit: int | Omit = omit,
        meter_type: str | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[FeatureListFeaturesResponse, AsyncMyCursorIDPage[FeatureListFeaturesResponse]]:
        """
        Retrieves a paginated list of features in the environment.

        Args:
          id: Filter by entity ID

          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          feature_type: Filter by feature type. Supports comma-separated values for multiple types

          limit: Maximum number of items to return

          meter_type: Filter by meter type. Supports comma-separated values for multiple types

          status: Filter by feature status. Supports comma-separated values for multiple statuses

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/features",
            page=AsyncMyCursorIDPage[FeatureListFeaturesResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "feature_type": feature_type,
                        "limit": limit,
                        "meter_type": meter_type,
                        "status": status,
                    },
                    feature_list_features_params.FeatureListFeaturesParams,
                ),
            ),
            model=FeatureListFeaturesResponse,
        )

    async def retrieve_feature(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feature:
        """
        Retrieves a feature by its unique identifier.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/v1/features/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feature,
        )

    async def unarchive_feature(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feature:
        """
        Restores an archived feature, allowing it to be used in entitlements again.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/features/{id}/unarchive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feature,
        )

    async def update_feature(
        self,
        id: str,
        *,
        description: str | Omit = omit,
        display_name: str | Omit = omit,
        enum_configuration: Iterable[feature_update_feature_params.EnumConfiguration] | Omit = omit,
        feature_units: str | Omit = omit,
        feature_units_plural: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        meter: feature_update_feature_params.Meter | Omit = omit,
        unit_transformation: Optional[feature_update_feature_params.UnitTransformation] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feature:
        """
        Updates an existing feature's properties such as display name, description, and
        configuration.

        Args:
          description: The description for the feature

          display_name: The display name for the feature

          enum_configuration: The configuration data for the feature

          feature_units: The units for the feature

          feature_units_plural: The plural units for the feature

          metadata: The additional metadata for the feature

          unit_transformation: Unit transformation to be applied to the reported usage

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/api/v1/features/{id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "display_name": display_name,
                    "enum_configuration": enum_configuration,
                    "feature_units": feature_units,
                    "feature_units_plural": feature_units_plural,
                    "metadata": metadata,
                    "meter": meter,
                    "unit_transformation": unit_transformation,
                },
                feature_update_feature_params.FeatureUpdateFeatureParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feature,
        )


class FeaturesResourceWithRawResponse:
    def __init__(self, features: FeaturesResource) -> None:
        self._features = features

        self.archive_feature = to_raw_response_wrapper(
            features.archive_feature,
        )
        self.create_feature = to_raw_response_wrapper(
            features.create_feature,
        )
        self.list_features = to_raw_response_wrapper(
            features.list_features,
        )
        self.retrieve_feature = to_raw_response_wrapper(
            features.retrieve_feature,
        )
        self.unarchive_feature = to_raw_response_wrapper(
            features.unarchive_feature,
        )
        self.update_feature = to_raw_response_wrapper(
            features.update_feature,
        )


class AsyncFeaturesResourceWithRawResponse:
    def __init__(self, features: AsyncFeaturesResource) -> None:
        self._features = features

        self.archive_feature = async_to_raw_response_wrapper(
            features.archive_feature,
        )
        self.create_feature = async_to_raw_response_wrapper(
            features.create_feature,
        )
        self.list_features = async_to_raw_response_wrapper(
            features.list_features,
        )
        self.retrieve_feature = async_to_raw_response_wrapper(
            features.retrieve_feature,
        )
        self.unarchive_feature = async_to_raw_response_wrapper(
            features.unarchive_feature,
        )
        self.update_feature = async_to_raw_response_wrapper(
            features.update_feature,
        )


class FeaturesResourceWithStreamingResponse:
    def __init__(self, features: FeaturesResource) -> None:
        self._features = features

        self.archive_feature = to_streamed_response_wrapper(
            features.archive_feature,
        )
        self.create_feature = to_streamed_response_wrapper(
            features.create_feature,
        )
        self.list_features = to_streamed_response_wrapper(
            features.list_features,
        )
        self.retrieve_feature = to_streamed_response_wrapper(
            features.retrieve_feature,
        )
        self.unarchive_feature = to_streamed_response_wrapper(
            features.unarchive_feature,
        )
        self.update_feature = to_streamed_response_wrapper(
            features.update_feature,
        )


class AsyncFeaturesResourceWithStreamingResponse:
    def __init__(self, features: AsyncFeaturesResource) -> None:
        self._features = features

        self.archive_feature = async_to_streamed_response_wrapper(
            features.archive_feature,
        )
        self.create_feature = async_to_streamed_response_wrapper(
            features.create_feature,
        )
        self.list_features = async_to_streamed_response_wrapper(
            features.list_features,
        )
        self.retrieve_feature = async_to_streamed_response_wrapper(
            features.retrieve_feature,
        )
        self.unarchive_feature = async_to_streamed_response_wrapper(
            features.unarchive_feature,
        )
        self.update_feature = async_to_streamed_response_wrapper(
            features.update_feature,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["EventReportResponse"]


class EventReportResponse(BaseModel):
    """Response object"""

    data: object
    """
    Empty success response confirming that events were successfully ingested and
    queued for processing by Stigg's metering system.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["PlanUpdateParams", "DefaultTrialConfig", "DefaultTrialConfigBudget"]


class PlanUpdateParams(TypedDict, total=False):
    billing_id: Annotated[Optional[str], PropertyInfo(alias="billingId")]
    """The unique identifier for the entity in the billing provider"""

    compatible_addon_ids: Annotated[Optional[SequenceNotStr[str]], PropertyInfo(alias="compatibleAddonIds")]

    default_trial_config: Annotated[Optional[DefaultTrialConfig], PropertyInfo(alias="defaultTrialConfig")]
    """Default trial configuration for the plan"""

    description: Optional[str]
    """The description of the package"""

    display_name: Annotated[str, PropertyInfo(alias="displayName")]
    """The display name of the package"""

    metadata: Dict[str, str]
    """Metadata associated with the entity"""

    parent_plan_id: Annotated[Optional[str], PropertyInfo(alias="parentPlanId")]
    """The ID of the parent plan, if applicable"""


class DefaultTrialConfigBudget(TypedDict, total=False):
    """Budget configuration for the trial"""

    has_soft_limit: Required[Annotated[bool, PropertyInfo(alias="hasSoftLimit")]]
    """Whether the budget limit is a soft limit (allows overage) or hard limit"""

    limit: Required[float]
    """The budget limit amount"""


class DefaultTrialConfig(TypedDict, total=False):
    """Default trial configuration for the plan"""

    duration: Required[float]
    """The duration of the trial in the specified units"""

    units: Required[Literal["DAY", "MONTH"]]
    """The time unit for the trial duration (DAY or MONTH)"""

    budget: Optional[DefaultTrialConfigBudget]
    """Budget configuration for the trial"""

    trial_end_behavior: Annotated[
        Optional[Literal["CONVERT_TO_PAID", "CANCEL_SUBSCRIPTION"]], PropertyInfo(alias="trialEndBehavior")
    ]
    """Behavior when the trial ends (CONVERT_TO_PAID or CANCEL_SUBSCRIPTION)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import TYPE_CHECKING, Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["Subscription", "Data", "DataPrice"]


class DataPrice(BaseModel):
    id: str
    """Price ID"""

    created_at: str = FieldInfo(alias="createdAt")
    """Creation timestamp"""

    updated_at: str = FieldInfo(alias="updatedAt")
    """Last update timestamp"""

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class Data(BaseModel):
    """Customer subscription to a plan"""

    id: str
    """Subscription ID"""

    billing_id: Optional[str] = FieldInfo(alias="billingId", default=None)
    """Billing ID"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Created at"""

    customer_id: str = FieldInfo(alias="customerId")
    """Customer ID"""

    payment_collection: Literal["NOT_REQUIRED", "PROCESSING", "FAILED", "ACTION_REQUIRED"] = FieldInfo(
        alias="paymentCollection"
    )
    """Payment collection"""

    plan_id: str = FieldInfo(alias="planId")
    """Plan ID"""

    pricing_type: Literal["FREE", "PAID", "CUSTOM"] = FieldInfo(alias="pricingType")
    """Pricing type"""

    start_date: datetime = FieldInfo(alias="startDate")
    """Subscription start date"""

    status: Literal["PAYMENT_PENDING", "ACTIVE", "EXPIRED", "IN_TRIAL", "CANCELED", "NOT_STARTED"]
    """Subscription status"""

    cancellation_date: Optional[datetime] = FieldInfo(alias="cancellationDate", default=None)
    """Subscription cancellation date"""

    cancel_reason: Optional[
        Literal[
            "UPGRADE_OR_DOWNGRADE",
            "CANCELLED_BY_BILLING",
            "EXPIRED",
            "DETACH_BILLING",
            "TRIAL_ENDED",
            "Immediate",
            "TRIAL_CONVERTED",
            "PENDING_PAYMENT_EXPIRED",
            "ScheduledCancellation",
            "CustomerArchived",
            "AutoCancellationRule",
        ]
    ] = FieldInfo(alias="cancelReason", default=None)
    """Subscription cancel reason"""

    current_billing_period_end: Optional[datetime] = FieldInfo(alias="currentBillingPeriodEnd", default=None)
    """End of the current billing period"""

    current_billing_period_start: Optional[datetime] = FieldInfo(alias="currentBillingPeriodStart", default=None)
    """Start of the current billing period"""

    effective_end_date: Optional[datetime] = FieldInfo(alias="effectiveEndDate", default=None)
    """Subscription effective end date"""

    end_date: Optional[datetime] = FieldInfo(alias="endDate", default=None)
    """Subscription end date"""

    metadata: Optional[Dict[str, str]] = None
    """Additional metadata for the subscription"""

    paying_customer_id: Optional[str] = FieldInfo(alias="payingCustomerId", default=None)
    """Paying customer ID for delegated billing"""

    payment_collection_method: Optional[Literal["CHARGE", "INVOICE", "NONE"]] = FieldInfo(
        alias="paymentCollectionMethod", default=None
    )
    """The method used to collect payments for a subscription"""

    prices: Optional[List[DataPrice]] = None

    resource_id: Optional[str] = FieldInfo(alias="resourceId", default=None)
    """Resource ID"""

    trial_end_date: Optional[datetime] = FieldInfo(alias="trialEndDate", default=None)
    """Subscription trial end date"""


class Subscription(BaseModel):
    """Response object"""

    data: Data
    """Customer subscription to a plan"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .plans import (
    PlansResource,
    AsyncPlansResource,
    PlansResourceWithRawResponse,
    AsyncPlansResourceWithRawResponse,
    PlansResourceWithStreamingResponse,
    AsyncPlansResourceWithStreamingResponse,
)
from .entitlements import (
    EntitlementsResource,
    AsyncEntitlementsResource,
    EntitlementsResourceWithRawResponse,
    AsyncEntitlementsResourceWithRawResponse,
    EntitlementsResourceWithStreamingResponse,
    AsyncEntitlementsResourceWithStreamingResponse,
)

__all__ = [
    "EntitlementsResource",
    "AsyncEntitlementsResource",
    "EntitlementsResourceWithRawResponse",
    "AsyncEntitlementsResourceWithRawResponse",
    "EntitlementsResourceWithStreamingResponse",
    "AsyncEntitlementsResourceWithStreamingResponse",
    "PlansResource",
    "AsyncPlansResource",
    "PlansResourceWithRawResponse",
    "AsyncPlansResourceWithRawResponse",
    "PlansResourceWithStreamingResponse",
    "AsyncPlansResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import datetime

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import usage_report_params, usage_history_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.v1.usage_report_response import UsageReportResponse
from ...types.v1.usage_history_response import UsageHistoryResponse

__all__ = ["UsageResource", "AsyncUsageResource"]


class UsageResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return UsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return UsageResourceWithStreamingResponse(self)

    def history(
        self,
        feature_id: str,
        *,
        customer_id: str,
        start_date: Union[str, datetime],
        end_date: Union[str, datetime] | Omit = omit,
        group_by: str | Omit = omit,
        resource_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageHistoryResponse:
        """
        Retrieves historical usage data for a customer's metered feature over time.

        Args:
          start_date: The start date of the range

          end_date: The end date of the range

          resource_id: Resource id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        if not feature_id:
            raise ValueError(f"Expected a non-empty value for `feature_id` but received {feature_id!r}")
        return self._get(
            f"/api/v1/usage/{customer_id}/history/{feature_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "start_date": start_date,
                        "end_date": end_date,
                        "group_by": group_by,
                        "resource_id": resource_id,
                    },
                    usage_history_params.UsageHistoryParams,
                ),
            ),
            cast_to=UsageHistoryResponse,
        )

    def report(
        self,
        *,
        usages: Iterable[usage_report_params.Usage],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageReportResponse:
        """Reports usage measurements for metered features.

        The reported usage is used to
        track, limit, and bill customer consumption.

        Args:
          usages: A list of usage reports to be submitted in bulk

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/usage",
            body=maybe_transform({"usages": usages}, usage_report_params.UsageReportParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageReportResponse,
        )


class AsyncUsageResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncUsageResourceWithStreamingResponse(self)

    async def history(
        self,
        feature_id: str,
        *,
        customer_id: str,
        start_date: Union[str, datetime],
        end_date: Union[str, datetime] | Omit = omit,
        group_by: str | Omit = omit,
        resource_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageHistoryResponse:
        """
        Retrieves historical usage data for a customer's metered feature over time.

        Args:
          start_date: The start date of the range

          end_date: The end date of the range

          resource_id: Resource id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        if not feature_id:
            raise ValueError(f"Expected a non-empty value for `feature_id` but received {feature_id!r}")
        return await self._get(
            f"/api/v1/usage/{customer_id}/history/{feature_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "start_date": start_date,
                        "end_date": end_date,
                        "group_by": group_by,
                        "resource_id": resource_id,
                    },
                    usage_history_params.UsageHistoryParams,
                ),
            ),
            cast_to=UsageHistoryResponse,
        )

    async def report(
        self,
        *,
        usages: Iterable[usage_report_params.Usage],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageReportResponse:
        """Reports usage measurements for metered features.

        The reported usage is used to
        track, limit, and bill customer consumption.

        Args:
          usages: A list of usage reports to be submitted in bulk

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/usage",
            body=await async_maybe_transform({"usages": usages}, usage_report_params.UsageReportParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageReportResponse,
        )


class UsageResourceWithRawResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.history = to_raw_response_wrapper(
            usage.history,
        )
        self.report = to_raw_response_wrapper(
            usage.report,
        )


class AsyncUsageResourceWithRawResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.history = async_to_raw_response_wrapper(
            usage.history,
        )
        self.report = async_to_raw_response_wrapper(
            usage.report,
        )


class UsageResourceWithStreamingResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.history = to_streamed_response_wrapper(
            usage.history,
        )
        self.report = to_streamed_response_wrapper(
            usage.report,
        )


class AsyncUsageResourceWithStreamingResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.history = async_to_streamed_response_wrapper(
            usage.history,
        )
        self.report = async_to_streamed_response_wrapper(
            usage.report,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.subscriptions.cancel_subscription import CancelSubscription

__all__ = ["FutureUpdateResource", "AsyncFutureUpdateResource"]


class FutureUpdateResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FutureUpdateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return FutureUpdateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FutureUpdateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return FutureUpdateResourceWithStreamingResponse(self)

    def cancel_pending_payment(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CancelSubscription:
        """
        Cancels a subscription update that is pending payment completion.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/api/v1/subscriptions/{id}/future-update/pending-payment",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CancelSubscription,
        )

    def cancel_schedule(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CancelSubscription:
        """
        Cancels a scheduled subscription update, such as a future downgrade or plan
        change.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/api/v1/subscriptions/{id}/future-update/schedule",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CancelSubscription,
        )


class AsyncFutureUpdateResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFutureUpdateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFutureUpdateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFutureUpdateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncFutureUpdateResourceWithStreamingResponse(self)

    async def cancel_pending_payment(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CancelSubscription:
        """
        Cancels a subscription update that is pending payment completion.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/api/v1/subscriptions/{id}/future-update/pending-payment",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CancelSubscription,
        )

    async def cancel_schedule(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CancelSubscription:
        """
        Cancels a scheduled subscription update, such as a future downgrade or plan
        change.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/api/v1/subscriptions/{id}/future-update/schedule",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CancelSubscription,
        )


class FutureUpdateResourceWithRawResponse:
    def __init__(self, future_update: FutureUpdateResource) -> None:
        self._future_update = future_update

        self.cancel_pending_payment = to_raw_response_wrapper(
            future_update.cancel_pending_payment,
        )
        self.cancel_schedule = to_raw_response_wrapper(
            future_update.cancel_schedule,
        )


class AsyncFutureUpdateResourceWithRawResponse:
    def __init__(self, future_update: AsyncFutureUpdateResource) -> None:
        self._future_update = future_update

        self.cancel_pending_payment = async_to_raw_response_wrapper(
            future_update.cancel_pending_payment,
        )
        self.cancel_schedule = async_to_raw_response_wrapper(
            future_update.cancel_schedule,
        )


class FutureUpdateResourceWithStreamingResponse:
    def __init__(self, future_update: FutureUpdateResource) -> None:
        self._future_update = future_update

        self.cancel_pending_payment = to_streamed_response_wrapper(
            future_update.cancel_pending_payment,
        )
        self.cancel_schedule = to_streamed_response_wrapper(
            future_update.cancel_schedule,
        )


class AsyncFutureUpdateResourceWithStreamingResponse:
    def __init__(self, future_update: AsyncFutureUpdateResource) -> None:
        self._future_update = future_update

        self.cancel_pending_payment = async_to_streamed_response_wrapper(
            future_update.cancel_pending_payment,
        )
        self.cancel_schedule = async_to_streamed_response_wrapper(
            future_update.cancel_schedule,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["UsageHistoryResponse", "Data", "DataMarker", "DataSeries", "DataSeriesPoint", "DataSeriesTag"]


class DataMarker(BaseModel):
    """Usage reset or change marker"""

    timestamp: datetime
    """Timestamp of the marker"""

    type: Literal["PERIODIC_RESET", "SUBSCRIPTION_CHANGE_RESET"]
    """Type of marker for a usage history point"""


class DataSeriesPoint(BaseModel):
    """Single usage data point"""

    is_reset_point: bool = FieldInfo(alias="isResetPoint")
    """
    Indicates whether there was usage reset in this point, see `markers` for details
    """

    timestamp: datetime
    """Timestamp of the usage history point"""

    value: float
    """Value of the usage history point"""


class DataSeriesTag(BaseModel):
    """Grouping tag key-value"""

    key: str
    """Key of the tag"""

    value: str
    """Value of the tag"""


class DataSeries(BaseModel):
    """Usage data points with tags"""

    points: List[DataSeriesPoint]
    """Points in the usage history series"""

    tags: List[DataSeriesTag]
    """Tags for the usage history series"""


class Data(BaseModel):
    """Historical usage time series"""

    markers: List[DataMarker]
    """Markers for events that affecting feature usage"""

    series: List[DataSeries]
    """Series of usage history"""


class UsageHistoryResponse(BaseModel):
    """Response object"""

    data: Data
    """Historical usage time series"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import event_report_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.v1.event_report_response import EventReportResponse

__all__ = ["EventsResource", "AsyncEventsResource"]


class EventsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return EventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return EventsResourceWithStreamingResponse(self)

    def report(
        self,
        *,
        events: Iterable[event_report_params.Event],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EventReportResponse:
        """Reports raw usage events for event-based metering.

        Events are ingested
        asynchronously and aggregated into usage totals.

        Args:
          events: A list of usage events to report

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/events",
            body=maybe_transform({"events": events}, event_report_params.EventReportParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventReportResponse,
        )


class AsyncEventsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncEventsResourceWithStreamingResponse(self)

    async def report(
        self,
        *,
        events: Iterable[event_report_params.Event],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EventReportResponse:
        """Reports raw usage events for event-based metering.

        Events are ingested
        asynchronously and aggregated into usage totals.

        Args:
          events: A list of usage events to report

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/events",
            body=await async_maybe_transform({"events": events}, event_report_params.EventReportParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventReportResponse,
        )


class EventsResourceWithRawResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.report = to_raw_response_wrapper(
            events.report,
        )


class AsyncEventsResourceWithRawResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.report = async_to_raw_response_wrapper(
            events.report,
        )


class EventsResourceWithStreamingResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.report = to_streamed_response_wrapper(
            events.report,
        )


class AsyncEventsResourceWithStreamingResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.report = async_to_streamed_response_wrapper(
            events.report,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ProductListProductsResponse", "ProductSettings"]


class ProductSettings(BaseModel):
    """Product behavior settings for subscription lifecycle management."""

    subscription_cancellation_time: Literal["END_OF_BILLING_PERIOD", "IMMEDIATE", "SPECIFIC_DATE"] = FieldInfo(
        alias="subscriptionCancellationTime"
    )
    """Time when the subscription will be cancelled"""

    subscription_end_setup: Literal["DOWNGRADE_TO_FREE", "CANCEL_SUBSCRIPTION"] = FieldInfo(
        alias="subscriptionEndSetup"
    )
    """Setup for the end of the subscription"""

    subscription_start_setup: Literal["PLAN_SELECTION", "TRIAL_PERIOD", "FREE_PLAN"] = FieldInfo(
        alias="subscriptionStartSetup"
    )
    """Setup for the start of the subscription"""

    downgrade_plan_id: Optional[str] = FieldInfo(alias="downgradePlanId", default=None)
    """ID of the plan to downgrade to at the end of the billing period"""

    prorate_at_end_of_billing_period: Optional[bool] = FieldInfo(alias="prorateAtEndOfBillingPeriod", default=None)
    """
    Indicates if the subscription should be prorated at the end of the billing
    period
    """

    subscription_start_plan_id: Optional[str] = FieldInfo(alias="subscriptionStartPlanId", default=None)
    """ID of the plan to start the subscription with"""


class ProductListProductsResponse(BaseModel):
    """Product configuration object"""

    id: str
    """The unique identifier for the entity"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp of when the record was created"""

    description: Optional[str] = None
    """Description of the product"""

    display_name: str = FieldInfo(alias="displayName")
    """Display name of the product"""

    metadata: Dict[str, str]
    """Metadata associated with the entity"""

    multiple_subscriptions: bool = FieldInfo(alias="multipleSubscriptions")
    """Indicates if multiple subscriptions to this product are allowed"""

    status: Literal["PUBLISHED", "ARCHIVED"]
    """The status of the product"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Timestamp of when the record was last updated"""

    product_settings: Optional[ProductSettings] = FieldInfo(alias="productSettings", default=None)
    """Product behavior settings for subscription lifecycle management."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["CustomerResponse", "Data", "DataDefaultPaymentMethod", "DataIntegration"]


class DataDefaultPaymentMethod(BaseModel):
    """The default payment method details"""

    billing_id: Optional[str] = FieldInfo(alias="billingId", default=None)
    """The default payment method id"""

    card_expiry_month: Optional[float] = FieldInfo(alias="cardExpiryMonth", default=None)
    """The expiration month of the default payment method"""

    card_expiry_year: Optional[float] = FieldInfo(alias="cardExpiryYear", default=None)
    """The expiration year of the default payment method"""

    card_last4_digits: Optional[str] = FieldInfo(alias="cardLast4Digits", default=None)
    """The last 4 digits of the default payment method"""

    type: Literal["CARD", "BANK", "CASH_APP"]
    """The default payment method type"""


class DataIntegration(BaseModel):
    """External billing or CRM integration link"""

    id: str
    """Integration details"""

    synced_entity_id: Optional[str] = FieldInfo(alias="syncedEntityId", default=None)
    """Synced entity id"""

    vendor_identifier: Literal[
        "AUTH0",
        "ZUORA",
        "STRIPE",
        "HUBSPOT",
        "AWS_MARKETPLACE",
        "SNOWFLAKE",
        "SALESFORCE",
        "BIG_QUERY",
        "OPEN_FGA",
        "APP_STORE",
    ] = FieldInfo(alias="vendorIdentifier")
    """The vendor identifier of integration"""


class Data(BaseModel):
    """A customer can be either an organization or an individual"""

    id: str
    """Customer slug"""

    archived_at: Optional[datetime] = FieldInfo(alias="archivedAt", default=None)
    """Timestamp of when the record was deleted"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp of when the record was created"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Timestamp of when the record was last updated"""

    coupon_id: Optional[str] = FieldInfo(alias="couponId", default=None)
    """Customer level coupon"""

    default_payment_method: Optional[DataDefaultPaymentMethod] = FieldInfo(alias="defaultPaymentMethod", default=None)
    """The default payment method details"""

    email: Optional[str] = None
    """The email of the customer"""

    integrations: Optional[List[DataIntegration]] = None
    """List of integrations"""

    metadata: Optional[Dict[str, str]] = None
    """Additional metadata"""

    name: Optional[str] = None
    """The name of the customer"""


class CustomerResponse(BaseModel):
    """Response object"""

    data: Data
    """A customer can be either an organization or an individual"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = [
    "SubscriptionPreviewParams",
    "Addon",
    "AppliedCoupon",
    "AppliedCouponConfiguration",
    "AppliedCouponDiscount",
    "AppliedCouponDiscountAmountsOff",
    "BillableFeature",
    "BillingInformation",
    "BillingInformationBillingAddress",
    "BillingInformationTaxID",
    "Charge",
    "TrialOverrideConfiguration",
]


class SubscriptionPreviewParams(TypedDict, total=False):
    customer_id: Required[Annotated[str, PropertyInfo(alias="customerId")]]
    """Customer ID"""

    plan_id: Required[Annotated[str, PropertyInfo(alias="planId")]]
    """Plan ID"""

    addons: Iterable[Addon]
    """Addons to include"""

    applied_coupon: Annotated[AppliedCoupon, PropertyInfo(alias="appliedCoupon")]
    """Coupon or discount to apply"""

    billable_features: Annotated[Iterable[BillableFeature], PropertyInfo(alias="billableFeatures")]
    """Billable features with quantities"""

    billing_country_code: Annotated[str, PropertyInfo(alias="billingCountryCode")]
    """ISO 3166-1 country code for localization"""

    billing_information: Annotated[BillingInformation, PropertyInfo(alias="billingInformation")]
    """Billing and tax configuration"""

    billing_period: Annotated[Literal["MONTHLY", "ANNUALLY"], PropertyInfo(alias="billingPeriod")]
    """Billing period (MONTHLY or ANNUALLY)"""

    charges: Iterable[Charge]
    """One-time or recurring charges"""

    paying_customer_id: Annotated[str, PropertyInfo(alias="payingCustomerId")]
    """Paying customer ID for delegated billing"""

    resource_id: Annotated[str, PropertyInfo(alias="resourceId")]
    """Resource ID for multi-instance subscriptions"""

    schedule_strategy: Annotated[
        Literal["END_OF_BILLING_PERIOD", "END_OF_BILLING_MONTH", "IMMEDIATE"], PropertyInfo(alias="scheduleStrategy")
    ]
    """When to apply subscription changes"""

    start_date: Annotated[Union[str, datetime], PropertyInfo(alias="startDate", format="iso8601")]
    """Subscription start date"""

    trial_override_configuration: Annotated[
        TrialOverrideConfiguration, PropertyInfo(alias="trialOverrideConfiguration")
    ]
    """Trial period override settings"""

    unit_quantity: Annotated[float, PropertyInfo(alias="unitQuantity")]
    """Unit quantity for per-unit pricing"""


class Addon(TypedDict, total=False):
    """Addon configuration"""

    addon_id: Required[Annotated[str, PropertyInfo(alias="addonId")]]
    """Addon ID"""

    quantity: int
    """Number of addon instances"""


class AppliedCouponConfiguration(TypedDict, total=False):
    """Coupon timing configuration"""

    start_date: Annotated[Union[str, datetime], PropertyInfo(alias="startDate", format="iso8601")]
    """Coupon start date"""


class AppliedCouponDiscountAmountsOff(TypedDict, total=False):
    amount: Required[float]
    """The price amount"""

    currency: Required[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ]
    """The price currency"""


class AppliedCouponDiscount(TypedDict, total=False):
    """Ad-hoc discount configuration"""

    amounts_off: Annotated[Optional[Iterable[AppliedCouponDiscountAmountsOff]], PropertyInfo(alias="amountsOff")]
    """Fixed amounts off by currency"""

    description: str
    """Ad-hoc discount"""

    duration_in_months: Annotated[float, PropertyInfo(alias="durationInMonths")]
    """Duration in months"""

    name: str
    """Discount name"""

    percent_off: Annotated[float, PropertyInfo(alias="percentOff")]
    """Percentage discount"""


class AppliedCoupon(TypedDict, total=False):
    """Coupon or discount to apply"""

    billing_coupon_id: Annotated[str, PropertyInfo(alias="billingCouponId")]
    """Billing provider coupon ID"""

    configuration: AppliedCouponConfiguration
    """Coupon timing configuration"""

    coupon_id: Annotated[str, PropertyInfo(alias="couponId")]
    """Stigg coupon ID"""

    discount: AppliedCouponDiscount
    """Ad-hoc discount configuration"""

    promotion_code: Annotated[str, PropertyInfo(alias="promotionCode")]
    """Promotion code to apply"""


class BillableFeature(TypedDict, total=False):
    """Feature with quantity"""

    feature_id: Required[Annotated[str, PropertyInfo(alias="featureId")]]
    """Feature ID"""

    quantity: Required[float]
    """Quantity of feature units"""


class BillingInformationBillingAddress(TypedDict, total=False):
    """Billing address"""

    city: str

    country: str

    line1: str

    line2: str

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]

    state: str


class BillingInformationTaxID(TypedDict, total=False):
    """Tax exemption identifier"""

    type: Required[str]
    """Tax exemption type (e.g., vat, gst)"""

    value: Required[str]
    """Tax exemption identifier value"""


class BillingInformation(TypedDict, total=False):
    """Billing and tax configuration"""

    billing_address: Annotated[BillingInformationBillingAddress, PropertyInfo(alias="billingAddress")]
    """Billing address"""

    charge_on_behalf_of_account: Annotated[str, PropertyInfo(alias="chargeOnBehalfOfAccount")]
    """Connected account ID for platform billing"""

    integration_id: Annotated[str, PropertyInfo(alias="integrationId")]
    """Billing integration ID"""

    invoice_days_until_due: Annotated[float, PropertyInfo(alias="invoiceDaysUntilDue")]
    """Days until invoice is due"""

    is_backdated: Annotated[bool, PropertyInfo(alias="isBackdated")]
    """Whether subscription is backdated"""

    is_invoice_paid: Annotated[bool, PropertyInfo(alias="isInvoicePaid")]
    """Whether invoice is already paid"""

    metadata: object
    """Additional billing metadata"""

    proration_behavior: Annotated[
        Literal["INVOICE_IMMEDIATELY", "CREATE_PRORATIONS", "NONE"], PropertyInfo(alias="prorationBehavior")
    ]
    """Proration behavior"""

    tax_ids: Annotated[Iterable[BillingInformationTaxID], PropertyInfo(alias="taxIds")]
    """Customer tax IDs"""

    tax_percentage: Annotated[float, PropertyInfo(alias="taxPercentage")]
    """Tax percentage to apply"""

    tax_rate_ids: Annotated[SequenceNotStr[str], PropertyInfo(alias="taxRateIds")]
    """Tax rate IDs from billing provider"""


class Charge(TypedDict, total=False):
    """Charge item"""

    id: Required[str]
    """Charge ID"""

    quantity: Required[float]
    """Charge quantity"""

    type: Required[Literal["FEATURE", "CREDIT"]]
    """Charge type"""


class TrialOverrideConfiguration(TypedDict, total=False):
    """Trial period override settings"""

    is_trial: Required[Annotated[bool, PropertyInfo(alias="isTrial")]]
    """Whether to start as trial"""

    trial_end_behavior: Annotated[
        Literal["CONVERT_TO_PAID", "CANCEL_SUBSCRIPTION"], PropertyInfo(alias="trialEndBehavior")
    ]
    """Behavior when trial ends"""

    trial_end_date: Annotated[Union[str, datetime], PropertyInfo(alias="trialEndDate", format="iso8601")]
    """Trial end date"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["UsageReportResponse", "Data"]


class Data(BaseModel):
    """Recorded usage with period info"""

    id: str
    """Unique identifier for the entity"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp of when the record was created"""

    customer_id: str = FieldInfo(alias="customerId")
    """Customer id"""

    feature_id: str = FieldInfo(alias="featureId")
    """Feature id"""

    timestamp: datetime
    """Timestamp"""

    value: float
    """The usage measurement record"""

    current_usage: Optional[float] = FieldInfo(alias="currentUsage", default=None)
    """The current measured usage value"""

    next_reset_date: Optional[datetime] = FieldInfo(alias="nextResetDate", default=None)
    """The date when the next usage reset will occur"""

    resource_id: Optional[str] = FieldInfo(alias="resourceId", default=None)
    """Resource id"""

    usage_period_end: Optional[datetime] = FieldInfo(alias="usagePeriodEnd", default=None)
    """
    The end date of the usage period in which this measurement resides (for
    entitlements with a reset period)
    """

    usage_period_start: Optional[datetime] = FieldInfo(alias="usagePeriodStart", default=None)
    """
    The start date of the usage period in which this measurement resides (for
    entitlements with a reset period)
    """


class UsageReportResponse(BaseModel):
    """
    Response containing reported usage measurements with current usage values, period information, and reset dates for each measurement.
    """

    data: List[Data]
    """Array of usage measurements with current values and period info"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.subscriptions import usage_charge_usage_params
from ....types.v1.subscriptions.usage_sync_response import UsageSyncResponse
from ....types.v1.subscriptions.usage_charge_usage_response import UsageChargeUsageResponse

__all__ = ["UsageResource", "AsyncUsageResource"]


class UsageResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return UsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return UsageResourceWithStreamingResponse(self)

    def charge_usage(
        self,
        id: str,
        *,
        until_date: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageChargeUsageResponse:
        """
        Immediately charges usage for a subscription via the billing integration.
        Calculates usage since the last charge and creates an invoice.

        Args:
          until_date: Cutoff date for usage calculation. If not provided, the current time is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/subscriptions/{id}/usage/charge",
            body=maybe_transform({"until_date": until_date}, usage_charge_usage_params.UsageChargeUsageParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageChargeUsageResponse,
        )

    def sync(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageSyncResponse:
        """
        Triggers a usage sync for a subscription, reporting current usage to the billing
        provider.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/subscriptions/{id}/usage/sync",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageSyncResponse,
        )


class AsyncUsageResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUsageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncUsageResourceWithStreamingResponse(self)

    async def charge_usage(
        self,
        id: str,
        *,
        until_date: Union[str, datetime] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageChargeUsageResponse:
        """
        Immediately charges usage for a subscription via the billing integration.
        Calculates usage since the last charge and creates an invoice.

        Args:
          until_date: Cutoff date for usage calculation. If not provided, the current time is used.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/subscriptions/{id}/usage/charge",
            body=await async_maybe_transform(
                {"until_date": until_date}, usage_charge_usage_params.UsageChargeUsageParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageChargeUsageResponse,
        )

    async def sync(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageSyncResponse:
        """
        Triggers a usage sync for a subscription, reporting current usage to the billing
        provider.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/subscriptions/{id}/usage/sync",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UsageSyncResponse,
        )


class UsageResourceWithRawResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.charge_usage = to_raw_response_wrapper(
            usage.charge_usage,
        )
        self.sync = to_raw_response_wrapper(
            usage.sync,
        )


class AsyncUsageResourceWithRawResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.charge_usage = async_to_raw_response_wrapper(
            usage.charge_usage,
        )
        self.sync = async_to_raw_response_wrapper(
            usage.sync,
        )


class UsageResourceWithStreamingResponse:
    def __init__(self, usage: UsageResource) -> None:
        self._usage = usage

        self.charge_usage = to_streamed_response_wrapper(
            usage.charge_usage,
        )
        self.sync = to_streamed_response_wrapper(
            usage.sync,
        )


class AsyncUsageResourceWithStreamingResponse:
    def __init__(self, usage: AsyncUsageResource) -> None:
        self._usage = usage

        self.charge_usage = async_to_streamed_response_wrapper(
            usage.charge_usage,
        )
        self.sync = async_to_streamed_response_wrapper(
            usage.sync,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["PlanListResponse", "DefaultTrialConfig", "DefaultTrialConfigBudget", "Entitlement"]


class DefaultTrialConfigBudget(BaseModel):
    """Budget configuration for the trial"""

    has_soft_limit: bool = FieldInfo(alias="hasSoftLimit")
    """Whether the budget limit is a soft limit (allows overage) or hard limit"""

    limit: float
    """The budget limit amount"""


class DefaultTrialConfig(BaseModel):
    """Default trial configuration for the plan"""

    duration: float
    """The duration of the trial in the specified units"""

    units: Literal["DAY", "MONTH"]
    """The time unit for the trial duration (DAY or MONTH)"""

    budget: Optional[DefaultTrialConfigBudget] = None
    """Budget configuration for the trial"""

    trial_end_behavior: Optional[Literal["CONVERT_TO_PAID", "CANCEL_SUBSCRIPTION"]] = FieldInfo(
        alias="trialEndBehavior", default=None
    )
    """Behavior when the trial ends (CONVERT_TO_PAID or CANCEL_SUBSCRIPTION)"""


class Entitlement(BaseModel):
    """Entitlement reference with type and identifier"""

    id: str
    """The unique identifier for the entity"""

    type: Literal["FEATURE", "CREDIT"]


class PlanListResponse(BaseModel):
    """Plan configuration object"""

    id: str
    """The unique identifier for the entity"""

    billing_id: Optional[str] = FieldInfo(alias="billingId", default=None)
    """The unique identifier for the entity in the billing provider"""

    compatible_addon_ids: Optional[List[str]] = FieldInfo(alias="compatibleAddonIds", default=None)

    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp of when the record was created"""

    default_trial_config: Optional[DefaultTrialConfig] = FieldInfo(alias="defaultTrialConfig", default=None)
    """Default trial configuration for the plan"""

    description: Optional[str] = None
    """The description of the package"""

    display_name: str = FieldInfo(alias="displayName")
    """The display name of the package"""

    entitlements: List[Entitlement]
    """List of entitlements of the package"""

    is_latest: Optional[bool] = FieldInfo(alias="isLatest", default=None)
    """Indicates if the package is the latest version"""

    metadata: Dict[str, str]
    """Metadata associated with the entity"""

    parent_plan_id: Optional[str] = FieldInfo(alias="parentPlanId", default=None)
    """The ID of the parent plan, if applicable"""

    pricing_type: Optional[Literal["FREE", "PAID", "CUSTOM"]] = FieldInfo(alias="pricingType", default=None)
    """The pricing type of the package"""

    product_id: str = FieldInfo(alias="productId")
    """The product id of the package"""

    status: Literal["DRAFT", "PUBLISHED", "ARCHIVED"]
    """The status of the package"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Timestamp of when the record was last updated"""

    version_number: int = FieldInfo(alias="versionNumber")
    """The version number of the package"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .customers import (
    CustomersResource,
    AsyncCustomersResource,
    CustomersResourceWithRawResponse,
    AsyncCustomersResourceWithRawResponse,
    CustomersResourceWithStreamingResponse,
    AsyncCustomersResourceWithStreamingResponse,
)
from .payment_method import (
    PaymentMethodResource,
    AsyncPaymentMethodResource,
    PaymentMethodResourceWithRawResponse,
    AsyncPaymentMethodResourceWithRawResponse,
    PaymentMethodResourceWithStreamingResponse,
    AsyncPaymentMethodResourceWithStreamingResponse,
)
from .promotional_entitlements import (
    PromotionalEntitlementsResource,
    AsyncPromotionalEntitlementsResource,
    PromotionalEntitlementsResourceWithRawResponse,
    AsyncPromotionalEntitlementsResourceWithRawResponse,
    PromotionalEntitlementsResourceWithStreamingResponse,
    AsyncPromotionalEntitlementsResourceWithStreamingResponse,
)

__all__ = [
    "PaymentMethodResource",
    "AsyncPaymentMethodResource",
    "PaymentMethodResourceWithRawResponse",
    "AsyncPaymentMethodResourceWithRawResponse",
    "PaymentMethodResourceWithStreamingResponse",
    "AsyncPaymentMethodResourceWithStreamingResponse",
    "PromotionalEntitlementsResource",
    "AsyncPromotionalEntitlementsResource",
    "PromotionalEntitlementsResourceWithRawResponse",
    "AsyncPromotionalEntitlementsResourceWithRawResponse",
    "PromotionalEntitlementsResourceWithStreamingResponse",
    "AsyncPromotionalEntitlementsResourceWithStreamingResponse",
    "CustomersResource",
    "AsyncCustomersResource",
    "CustomersResourceWithRawResponse",
    "AsyncCustomersResourceWithRawResponse",
    "CustomersResourceWithStreamingResponse",
    "AsyncCustomersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["AddonListResponse", "Entitlement"]


class Entitlement(BaseModel):
    """Entitlement reference with type and identifier"""

    id: str
    """The unique identifier for the entity"""

    type: Literal["FEATURE", "CREDIT"]


class AddonListResponse(BaseModel):
    """Addon configuration object"""

    id: str
    """The unique identifier for the entity"""

    billing_id: Optional[str] = FieldInfo(alias="billingId", default=None)
    """The unique identifier for the entity in the billing provider"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp of when the record was created"""

    dependencies: Optional[List[str]] = None
    """List of addons the addon is dependant on"""

    description: Optional[str] = None
    """The description of the package"""

    display_name: str = FieldInfo(alias="displayName")
    """The display name of the package"""

    entitlements: List[Entitlement]
    """List of entitlements of the package"""

    is_latest: Optional[bool] = FieldInfo(alias="isLatest", default=None)
    """Indicates if the package is the latest version"""

    max_quantity: Optional[int] = FieldInfo(alias="maxQuantity", default=None)
    """The maximum quantity of this addon that can be added to a subscription"""

    metadata: Dict[str, str]
    """Metadata associated with the entity"""

    pricing_type: Optional[Literal["FREE", "PAID", "CUSTOM"]] = FieldInfo(alias="pricingType", default=None)
    """The pricing type of the package"""

    product_id: str = FieldInfo(alias="productId")
    """The product id of the package"""

    status: Literal["DRAFT", "PUBLISHED", "ARCHIVED"]
    """The status of the package"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Timestamp of when the record was last updated"""

    version_number: int = FieldInfo(alias="versionNumber")
    """The version number of the package"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from stigg import Stigg, AsyncStigg
from tests.utils import assert_matches_type
from stigg._utils import parse_datetime
from stigg.types.v1 import UsageReportResponse, UsageHistoryResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUsage:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_history(self, client: Stigg) -> None:
        usage = client.v1.usage.history(
            feature_id="featureId",
            customer_id="customerId",
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(UsageHistoryResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_history_with_all_params(self, client: Stigg) -> None:
        usage = client.v1.usage.history(
            feature_id="featureId",
            customer_id="customerId",
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            group_by="groupBy",
            resource_id="resourceId",
        )
        assert_matches_type(UsageHistoryResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_history(self, client: Stigg) -> None:
        response = client.v1.usage.with_raw_response.history(
            feature_id="featureId",
            customer_id="customerId",
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageHistoryResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_history(self, client: Stigg) -> None:
        with client.v1.usage.with_streaming_response.history(
            feature_id="featureId",
            customer_id="customerId",
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageHistoryResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_history(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.v1.usage.with_raw_response.history(
                feature_id="featureId",
                customer_id="",
                start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `feature_id` but received ''"):
            client.v1.usage.with_raw_response.history(
                feature_id="",
                customer_id="customerId",
                start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_report(self, client: Stigg) -> None:
        usage = client.v1.usage.report(
            usages=[
                {
                    "customer_id": "customerId",
                    "feature_id": "featureId",
                    "value": -9007199254740991,
                }
            ],
        )
        assert_matches_type(UsageReportResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_report(self, client: Stigg) -> None:
        response = client.v1.usage.with_raw_response.report(
            usages=[
                {
                    "customer_id": "customerId",
                    "feature_id": "featureId",
                    "value": -9007199254740991,
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageReportResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_report(self, client: Stigg) -> None:
        with client.v1.usage.with_streaming_response.report(
            usages=[
                {
                    "customer_id": "customerId",
                    "feature_id": "featureId",
                    "value": -9007199254740991,
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageReportResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncUsage:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_history(self, async_client: AsyncStigg) -> None:
        usage = await async_client.v1.usage.history(
            feature_id="featureId",
            customer_id="customerId",
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(UsageHistoryResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_history_with_all_params(self, async_client: AsyncStigg) -> None:
        usage = await async_client.v1.usage.history(
            feature_id="featureId",
            customer_id="customerId",
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            group_by="groupBy",
            resource_id="resourceId",
        )
        assert_matches_type(UsageHistoryResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_history(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.usage.with_raw_response.history(
            feature_id="featureId",
            customer_id="customerId",
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = await response.parse()
        assert_matches_type(UsageHistoryResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_history(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.usage.with_streaming_response.history(
            feature_id="featureId",
            customer_id="customerId",
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageHistoryResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_history(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.v1.usage.with_raw_response.history(
                feature_id="featureId",
                customer_id="",
                start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `feature_id` but received ''"):
            await async_client.v1.usage.with_raw_response.history(
                feature_id="",
                customer_id="customerId",
                start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_report(self, async_client: AsyncStigg) -> None:
        usage = await async_client.v1.usage.report(
            usages=[
                {
                    "customer_id": "customerId",
                    "feature_id": "featureId",
                    "value": -9007199254740991,
                }
            ],
        )
        assert_matches_type(UsageReportResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_report(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.usage.with_raw_response.report(
            usages=[
                {
                    "customer_id": "customerId",
                    "feature_id": "featureId",
                    "value": -9007199254740991,
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = await response.parse()
        assert_matches_type(UsageReportResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_report(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.usage.with_streaming_response.report(
            usages=[
                {
                    "customer_id": "customerId",
                    "feature_id": "featureId",
                    "value": -9007199254740991,
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageReportResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["SubscriptionMigrateParams"]


class SubscriptionMigrateParams(TypedDict, total=False):
    subscription_migration_time: Annotated[
        Literal["END_OF_BILLING_PERIOD", "IMMEDIATE"], PropertyInfo(alias="subscriptionMigrationTime")
    ]
    """When to migrate (immediate or period end)"""
